#!/usr/bin/env python
#-*- coding:utf-8 -*-
#==============================================================================
#  Copyright (C) 2010 Schaefer & Tobies SuC GmbH.
#  Author: Markus Grimm <mgr@schaefer-tobies.de>
#
#  This file is part of Pyontapi.
#
#  Pyontapi is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  Pyontapi is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with Pyontapi.  If not, see <http://www.gnu.org/licenses/>.
#
#==============================================================================

""" Generator for API commands according to ONTAPI version. """

import sys

from schtob.pyontapi import errors, types, system


def generate(filer):
    """ generate API commands for ``filer``'s version. """

    api = system.System(filer)
    typedefs = types.gen_typedefs(api)
    return get_api_command_packages(api, typedefs)


def get_api_command_packages(api, typedefs):
    """
    Get all api commands for ``filer``. Returns a dict containing all
    packages.
    """

    try:
        result = api.api_list()
    except errors.APIFailure:
        exc = sys.exc_info()[1]
        raise errors.OntapiVersionError(exc.errno,
                'Can not run system-api-list. Error was:\n%s' % exc.reason
        )

    try:
        result = api.api_get_elements([
            child['name'] for child in result['apis']
        ])
    except errors.APIFailure:
        exc = sys.exc_info()[1]
        raise errors.OntapiVersionError(exc.errno,
                    'Can not get api elements. Error was:\n%s' % exc.reason
        )

    packages = {}

    for child in result['api-entries']:
        name = child['name']
        elements = []
        if 'api-elements' in child:
            for element_info in child['api-elements']:
                info = {}
                for key, value in element_info.items():
                    info[key] = value
                var_type = info['type'].replace('[]', '')
                if var_type in types.GENERIC_TYPEDEFS:
                    var_type = types.GENERIC_TYPEDEFS[var_type]
                else:
                    var_type = typedefs[var_type].duplicate()
                typedef = types.TypeDef(info, var_type)
                if isinstance(var_type, types.NamedType):
                    var_type.set_output_value_for_childs(
                        typedef.is_optional
                    )
                elements.append(typedef)
        api_command = types.APICommand(name, elements)
        if not api_command.get_package() in packages:
            packages[api_command.get_package()] = []
        packages[api_command.get_package()].append(api_command)
    return packages
